from collections import Counter
import string


tipos_tesouro = string.ascii_uppercase[:string.ascii_uppercase.index('V') + 1]

#! Tabela 9.5
tesouro_aleatorio = [
    # Covil
    {
        'tipo': 'A',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 12000}),
        'po': {'chance': 2, 'rolamento': '2d6 * 1000'},
        'pp': {'chance': 2, 'rolamento': '1d6 * 1000'},
        'pc': {'chance': 1, 'rolamento': '1d6 * 1000'},
        'gemas': {'chance': 3, 'rolamento': '6d6'},
        'objetos_de_valor': {'chance': 3, 'rolamento': '6d6'},
        'itens_magicos': {'chance': 2, 'itens': Counter({'qualquer': 3})},
    },
    {
        'tipo': 'B',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 1400}),
        'po': {'chance': 1, 'rolamento': '1d3 * 1000'},
        'pp': {'chance': 1, 'rolamento': '1d6 * 1000'},
        'pc': {'chance': 3, 'rolamento': '1d8 * 1000'},
        'gemas': {'chance': 1, 'rolamento': '1d6'},
        'objetos_de_valor': {'chance': 1, 'rolamento': '1d6'},
        'itens_magicos': {'chance': 1, 'itens': Counter({'arma': 1})},
    },
    {
        'tipo': 'C',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 650}),
        'pp': {'chance': 2, 'rolamento': '1d4 * 1000'},
        'pc': {'chance': 2, 'rolamento': '1d12 * 1000'},
        'gemas': {'chance': 1, 'rolamento': '1d4'},
        'objetos_de_valor': {'chance': 1, 'rolamento': '1d4'},
        'itens_magicos': {'chance': 1, 'itens': Counter({'qualquer': 2})},
    },
    {
        'tipo': 'D',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 3400}),
        'po': {'chance': 3, 'rolamento': '1d6 * 1000'},
        'pp': {'chance': 1, 'rolamento': '1d12 * 1000'},
        'pc': {'chance': 1, 'rolamento': '1d8 * 1000'},
        'gemas': {'chance': 2, 'rolamento': '1d8'},
        'objetos_de_valor': {'chance': 2, 'rolamento': '1d8'},
        'itens_magicos': {'chance': 1, 'itens': Counter({'qualquer': 2, 'poção': 1})},
    },
    {
        'tipo': 'E',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 1800}),
        'po': {'chance': 1, 'rolamento': '1d8 * 1000'},
        'pp': {'chance': 2, 'rolamento': '1d12 * 1000'},
        'pc': {'chance': 1, 'rolamento': '1d10 * 1000'},
        'gemas': {'chance': 1, 'rolamento': '1d10'},
        'objetos_de_valor': {'chance': 1, 'rolamento': '1d10'},
        'itens_magicos': {'chance': 1, 'itens': Counter({'qualquer': 3, 'pergaminho': 1})},
    },
    {
        'tipo': 'F',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 4000}),
        'po': {'chance': 2, 'rolamento': '1d12 * 1000'},
        'pp': {'chance': 1, 'rolamento': '2d10 * 1000'},
        'gemas': {'chance': 1, 'rolamento': '2d12'},
        'objetos_de_valor': {'chance': 1, 'rolamento': '1d12'},
        'itens_magicos': {'chance': 2, 'itens': Counter({'poção': 1, 'pergaminho': 1, 'não arma': 3})},
    },
    {
        'tipo': 'G',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 14000}),
        'po': {'chance': 3, 'rolamento': '10d4 * 1000'},
        'gemas': {'chance': 1, 'rolamento': '3d6'},
        'objetos_de_valor': {'chance': 1, 'rolamento': '1d10'},
        'itens_magicos': {'chance': 2, 'itens': Counter({'qualquer': 4, 'pergaminho': 1})},
    },
    {
        'tipo': 'H',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 27000}),
        'po': {'chance': 3, 'rolamento': '10d6 * 1000'},
        'pp': {'chance': 3, 'rolamento': '1d10 * 1000'},
        'pc': {'chance': 1, 'rolamento': '3d8 * 1000'},
        'gemas': {'chance': 3, 'rolamento': '1d10'},
        'objetos_de_valor': {'chance': 3, 'rolamento': '10d4'},
        'itens_magicos': {'chance': 1, 'itens': Counter({'qualquer': 4, 'poção': 1, 'pergaminho': 1})},
    },
    {
        'tipo': 'I',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 2800}),
        'gemas': {'chance': 3, 'rolamento': '2d6'},
        'objetos_de_valor': {'chance': 3, 'rolamento': '2d6'},
        'itens_magicos': {'chance': 1, 'itens': Counter({'qualquer': 1})},
    },
    {
        'tipo': 'J',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 25}),
        'pp': {'chance': 1, 'rolamento': '1d3 * 1000'},
        'pc': {'chance': 1, 'rolamento': '1d4 * 1000'},
    },
    {
        'tipo': 'K',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 15}),
        'pp': {'chance': 1, 'rolamento': '1d2 * 1000'},
    },
    {
        'tipo': 'L',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 200}),
        'gemas': {'chance': 3, 'rolamento': '1d4'},
    },
    {
        'tipo': 'M',
        'categoria': 'covil',
        'tesouro_rapido': Counter({'po': 40000}),
        'po': {'chance': 5, 'rolamento': '8d10 * 1000'},
        'pp': {'chance': 3, 'rolamento': '10d6 * 1000'},
        'gemas': {'chance': 3, 'rolamento': '5d4'},
        'objetos_de_valor': {'chance': 2, 'rolamento': '2d6'},
    },
    {
        'tipo': 'N',
        'categoria': 'covil',
        'itens_magicos': {'chance': 2, 'itens': Counter({'poção': '2d4'})},
    },
    {
        'tipo': 'O',
        'categoria': 'covil',
        'itens_magicos': {'chance': 3, 'itens': Counter({'poção': '1d4'})},
    },
    # Individual
    {
        'tipo': 'P',
        'categoria': 'individual',
        'tesouro_rapido': Counter({'pp': 1}),
        'pc': {'chance': 6, 'rolamento': '3d8'},
    },
    {
        'tipo': 'Q',
        'categoria': 'individual',
        'tesouro_rapido': Counter({'po': 1}),
        'pp': {'chance': 6, 'rolamento': '3d6'},
    },
    {
        'tipo': 'R',
        'categoria': 'individual',
        'tesouro_rapido': Counter({'po': 3}),
        'po': {'chance': 6, 'rolamento': '1d6'},
        'equipamentos': {'chance': 2, 'rolamento': 1}
    },
    {
        'tipo': 'S',
        'categoria': 'individual',
        'tesouro_rapido': Counter({'po': 5}),
        'po': {'chance': 6, 'rolamento': '2d4'},
        'equipamentos': {'chance': 2, 'rolamento': 1}
    },
    {
        'tipo': 'T',
        'categoria': 'individual',
        'tesouro_rapido': Counter({'po': 17}),
        'po': {'chance': 6, 'rolamento': '1d6 * 5'},
        'equipamentos': {'chance': 2, 'rolamento': 2}
    },
    {
        'tipo': 'U',
        'categoria': 'individual',
        'tesouro_rapido': Counter({'po': 90}),
        'po': {'chance': 1, 'rolamento': '1d10'},
        'pp': {'chance': 1, 'rolamento': '1d10'},
        'pc': {'chance': 1, 'rolamento': '1d10'},
        'objetos_de_valor': {'chance': 1, 'rolamento': 1},
        'itens_magicos': {'chance': 1, 'itens': Counter({'qualquer': 1})},
        'equipamentos': {'chance': 1, 'rolamento': '1d4'}
    },
    {
        'tipo': 'V',
        'categoria': 'individual',
        'tesouro_rapido': Counter({'po': 175}),
        'po': {'chance': 2, 'rolamento': '1d10'},
        'pp': {'chance': 2, 'rolamento': '1d10'},
        'objetos_de_valor': {'chance': 1, 'rolamento': '1d4'},
        'itens_magicos': {'chance': 2, 'itens': Counter({'qualquer': 1})},
        'equipamentos': {'chance': 1, 'rolamento': '1d6'}
    }
]

#! Tabela 9.6
equipamentos_raridade = [
    ((2, 2), "raro"),
    ((3, 3), "raro"),
    ((4, 4), "incomum"),
    ((5, 5), "incomum"),
    ((6, 6), "comum"),
    ((7, 7), "comum"),
    ((8, 8), "comum"),
    ((9, 9), "comum"),
    ((10, 10), "incomum"),
    ((11, 11), "incomum"),
    ((12, 12), "raro")
]
equipamentos_tipos = [
    ((2, 2), {
        "comum": "símbolo divino",
        "incomum": "aljava (1d6 flechas)",
        "raro": "porta mapas"
    }),
    ((3, 3), {
        "comum": "saco de dormir",
        "incomum": "martelo",
        "raro": "pena e tinta"
    }),
    ((4, 4), {
        "comum": "ração de viagem (1d4)",
        "incomum": "óleo",
        "raro": "corrente"
    }),
    ((5, 5), {
        "comum": "pederneira",
        "incomum": "água benta",
        "raro": "algema"
    }),
    ((6, 6), {
        "comum": "corda de cânhamo (15m)",
        "incomum": "pá ou picareta",
        "raro": "giz"
    }),
    ((7, 7), {
        "comum": "tochas (1d4)",
        "incomum": "arpéu",
        "raro": "caixa pequena"
    }),
    ((8, 8), {
        "comum": "mochila",
        "incomum": "lamparina",
        "raro": "coberta de inverno"
    }),
    ((9, 9), {
        "comum": "odre",
        "incomum": "vela (1d4)",
        "raro": "espelho"
    }),
    ((10, 10), {
        "comum": "saco de estopa",
        "incomum": "cravos ou ganchos (1d4)",
        "raro": "cadeado"
    }),
    ((11, 11), {
        "comum": "traje de exploração",
        "incomum": "traje de inverno",
        "raro": "traje nobre"
    }),
    ((12, 12), {
        "comum": "ferramenta de ladrão",
        "incomum": "lanterna furta-fogo",
        "raro": "rede"
    })
]

#! Tabela 9.7
objetos_valor_raridade = [
    ((2, 3), "obras de arte"),
    ((4, 5), "utensílios"),
    ((6, 7), "mercadoria"),
    ((8, 9), "mercadoria"),
    ((10, 11), "louças"),
    ((12, 12), "joias")
]
objetos_valor_tipos = [
    ((2, 3), {
        "mercadoria": "peles de animais raros *",
        "louças": "objetos de vidro soprado",
        "utensílios": "religiosos de cobre",
        "obras de arte": "móveis com marchetaria *",
        "joias": "cordão de prata"
    }),
    ((4, 5), {
        "mercadoria": "objetos de marfim",
        "louças": "copos de vidro e com prata",
        "utensílios": "talheres de prata",
        "obras de arte": "tapeçaria fina *",
        "joias": "brincos de prata"
    }),
    ((6, 7), {
        "mercadoria": "sacas de especiaria *",
        "louças": "baixelas de louça",
        "utensílios": "candelabros de prata",
        "obras de arte": "livro raro",
        "joias": "bracelete de prata"
    }),
    ((8, 9), {
        "mercadoria": "sacas de incenso *",
        "louças": "baixelas de porcelana com ouro",
        "utensílios": "cutelaria fina",
        "obras de arte": "escultura *",
        "joias": "pingente de pedraria"
    }),
    ((10, 11), {
        "mercadoria": "tecidos nobres *",
        "louças": "vasos de porcelana",
        "utensílios": "cálices de ouro",
        "obras de arte": "tela pintada *",
        "joias": "camafeu de ouro"
    }),
    ((12, 12), {
        "mercadoria": "metros de fina seda *",
        "louças": "cálices de vidro com pedra",
        "utensílios": "religiosos de ouro",
        "obras de arte": "estatueta em bronze *",
        "joias": "tiara de pedra rara"
    })
]

#! Tabela 9.8
gemas = [
    ((2, 3), {"categoria": "preciosa", "valor": 500}),
    ((4, 5), {"categoria": "ornamental", "valor": 50}),
    ((6, 7), {"categoria": "decorativa", "valor": 10}),
    ((8, 9), {"categoria": "decorativa", "valor": 10}),
    ((10, 11), {"categoria": "semipreciosa", "valor": 100}),
    ((12, 12), {"categoria": "joia", "valor": 1000})
]
gema_modificador = [
    ((1, 2), {'qualidade':'bruta', 'modificador': 0.75}),
    ((3, 3), {'qualidade':'trincada', 'modificador': 0.5}),
    ((4, 6), {'qualidade':'lapidada', 'modificador': 1}),
]

#! Tabela 8.2
qualquer = [
    ((1, 20), "espada"),
    ((21, 30), "outra arma"),
    ((31, 40), "armadura"),
    ((41, 65), "poção"),
    ((66, 85), "pergaminho"),
    ((86, 90), "anel"),
    ((91, 95), "haste mágica"),
    ((96, 100), "itens mágicos gerais")
]
nao_arma = [
    ((1, 14), "armadura"),
    ((15, 50), "poção"),
    ((51, 85), "pergaminho"),
    ((86, 90), "anel"),
    ((91, 95), "haste mágica"),
    ((96, 100), "itens mágicos gerais")
]
arma = [
    ((1, 65), "espada"),
    ((66, 100), "outra arma")
]

#! Tabela 8.3
espada_tipo = [
    ((1, 79), "espada longa"),
    ((80, 89), "espada curta"),
    ((90, 94), "cimitarra"),
    ((95, 99), "espada bastarda"),
    ((100, 100), "montante")
]
espada_bonus = [
    ((1, 3), "-2 amaldiçoada"),
    ((4, 10), "-1 amaldiçoada"),
    ((11, 64), "+1"),
    ((65, 84), "+2"),
    ((85, 94), "+3 (+5% na escolha do talento)"),
    ((95, 99), "+4 (+10% na escolha do talento)"),
    ((100, 100), "+5 (+20% na escolha do talento)")
]
espada_talento = [
    ((1, 59), "Nenhum Talento"),
    ((60, 62), "Matadora de licantropos"),
    ((63, 65), "Matadora de orcs"),
    ((66, 68), "Matadora de mortos-vivos"),
    ((69, 71), "Matadora de usuários de magia"),
    ((72, 74), "Matadora de gigantes"),
    ((75, 77), "Matadora de regeneradores"),
    ((78, 79), "Matadora de dragões"),
    ((80, 81), "Matadora de extraplanares"),
    ((82, 83), "Defensora"),
    ((84, 85), "da Cura"),
    ((86, 87), "de Drenar Energia"),
    ((88, 89), "da Luz"),
    ((90, 91), "Flamejante"),
    ((92, 93), "Gélida"),
    ((94, 95), "da Respiração"),
    ((96, 97), "da Velocidade"),
    ((98, 99), "Vorpal"),
    ((100, 100), "Inteligente")
]

#! Tabela 8.4
espada_inteligente = [
    ((7, 7), {
        "comunicação": "empatia",
        "idiomas": None,
        "poderes de detecção": 1,
        "poderes maiores": None
    }),
    ((8, 8), {
        "comunicação": "empatia",
        "idiomas": None,
        "poderes de detecção": 2,
        "poderes maiores": None
    }),
    ((9, 9), {
        "comunicação": "empatia",
        "idiomas": None,
        "poderes de detecção": 3,
        "poderes maiores": None
    }),
    ((10, 10), {
        "comunicação": "fala",
        "idiomas": "1d3",
        "poderes de detecção": 3,
        "poderes maiores": None
    }),
    ((11, 11), {
        "comunicação": "fala",
        "idiomas": "1d4",
        "poderes de detecção": 3,
        "poderes maiores": None
    }),
    ((12, 12), {
        "comunicação": "fala",
        "idiomas": "1d6",
        "poderes de detecção": 3,
        "poderes maiores": 1
    })
]

#! Tabela 8.5
arma_tipo = [
    ((1, 3), "flecha"),
    ((4, 4), "arco curto"),
    ((5, 5), "arco longo"),
    ((6, 6), "besta de mão"),
    ((7, 7), "besta"),
    ((8, 9), "funda (2d10+4 unidades de munição)"),
    ((10, 19), "adaga"),
    ((20, 21), "alabarda"),
    ((22, 23), "azagaia"),
    ((24, 25), "bordão/cajado"),
    ((26, 30), "lança"),
    ((31, 40), "lança montada"),
    ((41, 50), "maça"),
    ((51, 60), "machado"),
    ((61, 65), "machado de arremesso"),
    ((66, 80), "machado de batalha"),
    ((81, 85), "mangual"),
    ((86, 95), "martelo de batalha"),
    ((96, 98), "pique"),
    ((99, 100), "porrete/clava")
]
arma_bonus = [
    ((1, 3), "-2 amaldiçoada (caótica)"),
    ((4, 10), "-1 amaldiçoada (caótica)"),
    ((11, 74), "+1"),
    ((75, 94), "+2"),
    ((95, 100), "+3 (+5% no teste de talento)")
]
arma_talento = [
    ((1, 90), "nenhum talento"),
    ((91, 100), "especial (por tipo de arma)")
]

#! Tabela 8.6
armadura_tipo = [
    ((1, 40), "escudo"),
    ((41, 45), "armadura acolchoada"),
    ((46, 50), "armadura de couro"),
    ((51, 55), "armadura de couro batido"),
    ((56, 85), "cota de malha"),
    ((86, 95), "armadura de placas"),
    ((96, 100), "armadura completa")
]
armadura_bonus = [
    ((1, 3), "-2 amaldiçoada"),
    ((4, 10), "-1 amaldiçoada"),
    ((11, 74), "+1"),
    ((75, 94), "+2"),
    ((95, 100), "+3 (+5% no teste de talento)")
]
armadura_talento = [
    ((1, 93), "nenhum talento"),
    ((94, 94), "da absorção"),
    ((95, 95), "da velocidade"),
    ((96, 96), "curadora"),
    ((97, 97), "da retribuição"),
    ((98, 98), "da invisibilidade"),
    ((99, 99), "da reflexão"),
    ((100, 100), "contra projéteis")
]

#! Tabela 8.7
pocoes = [
    ((1, 5), "poção amaldiçoada"),
    ((6, 15), "poção de placebo"),
    ((16, 50), "poção de cura"),
    ((51, 64), ["poção de controle de animais", "poção de controle de humanos", "poção de controle de plantas"]),
    ((65, 66), "poção da diminuição"),
    ((67, 68), "poção da forma gasosa"),
    ((69, 70), "poção da força gigante"),
    ((71, 72), "poção do crescimento"),
    ((73, 74), "poção da invisibilidade"),
    ((75, 76), "veneno"),
    ((77, 78), "antídoto"),
    ((79, 80), "poção de defesa"),
    ((81, 82), "poção da metamorfose"),
    ((83, 84), "poção da velocidade"),
    ((85, 86), "poção da clarividência"),
    ((87, 88), "poção da percepção extrassensorial"),
    ((89, 90), "poção da resistência ao fogo"),
    ((91, 92), "poção do voo"),
    ((93, 94), "poção do heroísmo"),
    ((95, 96), "poção de respirar na água"),
    ((97, 98), "poção da sorte"),
    ((99, 100), "poção do salto")
]

#! Tabela 8.8
pergaminhos = [
    ((1, 15), "pergaminho amaldiçoado"),
    ((16, 30), "pergaminho arcano (1 círculo)"),
    ((31, 40), "pergaminho arcano (3 círculos)"),
    ((41, 45), "pergaminho arcano (4 círculos)"),
    ((46, 48), "pergaminho arcano (7 círculos)"),
    ((49, 50), "pergaminho arcano (9 círculos)"),
    ((51, 60), "pergaminho divino (1 círculo)"),
    ((61, 63), "pergaminho divino (3 círculos)"),
    ((64, 65), "pergaminho divino (7 círculos)"),
    ((66, 67), "pergaminho de proteção ao caos"),
    ((68, 69), "pergaminho de proteção à ordem"),
    ((70, 71), "pergaminho de proteção à magia"),
    ((72, 73), "pergaminho de proteção à mortos-vivos"),
    ((74, 74), "pergaminho de proteção à licantropos"),
    ((75, 75), "pergaminho de proteção à elementais"),
    ((76, 78), "mapa do tesouro (tipo a)"),
    ((79, 81), "mapa do tesouro (tipo b)"),
    ((82, 84), "mapa do tesouro (tipo c)"),
    ((85, 87), "mapa do tesouro (tipo d)"),
    ((88, 90), "mapa do tesouro (tipo e)"),
    ((91, 93), "mapa do tesouro (tipo f)"),
    ((94, 96), "mapa do tesouro (tipo g)"),
    ((97, 99), "mapa do tesouro (tipo h)"),
    ((100, 100), "mapa do tesouro (tipo m)")
]

#! Tabela 8.9
aneis = [
    ((1, 15), "anel amaldiçoado"),
    ((16, 45), "anel de proteção +1"),
    ((46, 65), "anel de proteção +2"),
    ((66, 68), "anel de proteção +3"),
    ((69, 69), "anel de proteção +4"),
    ((70, 71), "anel do controle de animais"),
    ((72, 73), "anel do controle de humanos"),
    ((74, 75), "anel do controle de plantas"),
    ((76, 77), "anel da regeneração"),
    ((78, 79), "anel da invisibilidade"),
    ((80, 81), "anel da resistência ao fogo"),
    ((82, 83), "anel da telecinesia"),
    ((84, 85), "anel de andar sobre as águas"),
    ((86, 87), "anel de refletir magias"),
    ((88, 89), "anel de armazenar magia"),
    ((90, 91), "anel da anti-ilusão"),
    ((92, 93), "anel da verdade"),
    ((94, 95), "anel do ouro de tolo"),
    ((96, 97), "anel da santidade"),
    ((98, 99), "anel da visão de raio-x"),
    ((100, 100), "anel do desejo")
]

#! Tabela 8.10
hastes = [
    ((1, 15), "haste amaldiçoada"),
    ((16, 22), "varinha de detecção de inimigos"),
    ((23, 30), "varinha de detecção de magia"),
    ((31, 37), "varinha de detecção de armadilhas"),
    ((38, 44), "varinha de paralisação"),
    ((45, 51), "varinha de bola de fogo"),
    ((52, 58), "varinha do medo"),
    ((59, 65), "varinha do congelamento"),
    ((66, 72), "varinha da ilusão"),
    ((73, 79), "varinha do relâmpago"),
    ((80, 86), "varinha da transformação"),
    ((87, 88), "cajado da cura"),
    ((89, 90), "cajado de ataque"),
    ((91, 92), "cajado da serpente"),
    ((93, 94), "cajado da anulação"),
    ((95, 96), "cajado do controle"),
    ((97, 97), "bastão do governante"),
    ((98, 98), "bastão do bloqueio"),
    ((99, 99), "bastão do armamento"),
    ((100, 100), "bastão do cancelamento")
]

#! Tabela 8.11
itens_magicos_gerais = [
    ((1, 4), "livro dos grandes feitos"),
    ((5, 8), "bestiário: o livro dos monstros"),
    ((9, 12), "grande livro da conjuração"),
    ((13, 16), "medalhão da PES"),
    ((17, 20), "camafeu do aprisionamento"),
    ((21, 24), "manto do deslocamento"),
    ((25, 28), "manto élfico"),
    ((29, 32), "bota da levitação"),
    ((33, 36), "bota élfica"),
    ((37, 40), "manoplas da força do ogro"),
    ((41, 44), "elmo da mudança de alinhamento (caótico)"),
    ((45, 48), "cinto da força do gigante"),
    ((49, 52), "elmo da telepatia"),
    ((53, 56), "tambores do pânico (caótico)"),
    ((57, 60), "trombeta da destruição (caótica)"),
    ((61, 64), "sacola devoradora (caótica)"),
    ((65, 68), "sacola guardiã"),
    ((69, 72), "buraco portátil"),
    ((73, 76), "corda da escalada"),
    ((77, 80), "vassoura de voo"),
    ((81, 84), "bola de cristal"),
    ((85, 88), "baralho das maravilhas (caótico)"),
    ((89, 92), "baralho da navegação planar"),
    ((93, 96), "garrafa do gênio"),
    ((97, 100), "tapete voador")
]

#! Talentos de armas
adagas = ['de prata', 'do retorno', 'crítica', 'flamejante']
armas_haste = ['do desarme', 'defensora', 'vigilante']
pique = ['do desarme', 'defensor', 'vigilante']
lanca = ['do retorno', 'do desarme', 'defensora']
cajado = ['defensor', 'curador', 'do desarme']
impacto = ['do retorno', 'da explosão', 'do voo', 'do desarme']
machado = ['crítico', 'da velocidade', 'de arremesso', 'matador']
machado_arremesso = ['crítico', 'da velocidade', 'do retorno', 'matador']
projeteis = ['do atordoamento', 'do desarme', 'penetrante', 'da explosão']
matador = ['de orcs', 'de mortos-vivos', 'de gigantes', 'de usuários de magia']
flecha = ['flecha de caça', 'flecha de guerra', 'virote', 'virote pequeno']
qtd_flecha = '2d6+6'
qtd_funda = '2d10+6'



from .converter_tipo import converter_para_numero, converter_para_texto
from .retorna_atributo import retorna_mod_atributo
from .atributo import AtributoBase, Atributo
from .moedas import converter_moedas, Moedas
from .rolagens import teste_atributo, jogada_protecao, jogada_ataque, x_em_d6



from .api.campanhas import campanhas
from .api.classes import classes
from .api.equipamentos import equipamentos
from .api.livros import livros
from .api.magias import magias
from .api.monstros import monstros
from .api.personagens import personagens
from .api.racas import racas

def filtrar_acesso_completo(variavel: str):
    lista = variavel

    if 'access' not in lista[0].keys():
        return lista

    return list(filter(lambda x: x.get('access') == 'complete', lista))


classes = filtrar_acesso_completo(classes)
equipamentos = filtrar_acesso_completo(equipamentos)
livros = livros
magias = filtrar_acesso_completo(magias)
monstros = filtrar_acesso_completo(monstros)
racas = filtrar_acesso_completo(racas)



class EntidadeBase:
    def __init__(self,
                 pv: int
                 ):
        self._pv: int = pv
        self._pv_atual: int = pv

        self.esta_morto: bool = False

    #? Propriedades
    @property
    def pv_atual(self):
        return self._pv_atual

    @pv_atual.setter
    def pv_atual(self, valor: int):
        self._pv_atual = valor
        self._pv_atual = 0 if self._pv_atual < 0 else self._pv_atual
        self._pv_atual = self._pv if self._pv_atual > self._pv else self._pv_atual

        self.esta_morto = True if self._pv_atual == 0 else False


    #? Métodos
    def ferir(self, valor: int):
        self.pv_atual -= valor

    def curar(self, valor: int):
        self.pv_atual += valor



from RPG import Dado, d20, rolar_dado_notacao
from collections import namedtuple

from .entidade import EntidadeBase
from ..helpers import converter_para_numero, converter_para_texto, jogada_ataque
from ..data import DATA, buscar
from ..utils import TesouroAleatorio

class Monstro(EntidadeBase):
    # Mapeamento de chaves do dicionário original para atributos em português
    mapa_de_atributos = {
        'name': 'nome',
        'xp': 'xp',
        'dv': 'dv',
        'dv_bonus': 'dv_bonus',
        'pv': 'pv',
        'ca': 'ca',
        'jp': 'jp',
        'mv': 'movimento',
        'mo': 'moral',
        'attacks': 'ataques',
        'alignment': 'alinhamento',
        'size': 'tamanho',
        'habitats': 'habitats',
        'type': 'tipo',
        'id': 'id',
        'url': 'url',
        'picture': 'imagem',
        'thumb_picture': 'miniatura',
        'treasure': 'tesouro',
        'treasure_lair': 'tesouro_covil',
        'encounters': 'encontros',
        'encounters_lair': 'encontros_covil',
        'variant': 'variacao',
        'variant_modifications': 'variante_modificadores',
        'access': 'acesso',
        'concept': 'conceito',
        'flavor': 'flavor',
        'fontes': 'fontes',
        'description': 'descricao',
        'mvc': 'movimento_cavando',
        'mve': 'movimento_escalando',
        'mvn': 'movimento_nadando',
        'mvo': 'movimento_outros',
        'mvv': 'movimento_voo',
    }

    def __init__(self, monstro: dict | str):
        self._dados = dict(monstro if isinstance(monstro, dict) else buscar(DATA.MONSTROS, monstro))

        for chave_original, nome_em_portugues in self.mapa_de_atributos.items():
            setattr(self, nome_em_portugues, self._dados.get(chave_original))


        # Estatísticas
        self.xp = converter_para_numero(self._dados.get('xp'))
        self.dv = self._retornar_dv(self._dados.get('dv'))
        self.dv_bonus = self._retornar_mod_dv(self._dados.get('dv_bonus'))
        self.pv = converter_para_numero(self._dados.get('pv'))
        self.ca = self._retornar_ca(self._dados.get('ca'))

        # Propriedades para alteração privada
        super().__init__(self.pv)

    #? Propriedades
    @property
    def dados(self):
        return dict(self._dados)


    def __repr__(self):
        return f"<Monstro(nome={self.nome})>"

    def __str__(self):
        ataques_str = 'Nenhum ataque'
        mod_dv = '' if not self.dv_bonus else self.dv_bonus
        mod_dv = f'+{mod_dv}' if mod_dv and mod_dv > 0 else mod_dv

        if self.ataques:
            ataques_str = ', '.join(
                a.get('text', 'Ataque desconhecido') for a in self.ataques)

        return (
            f"{self.nome} ({self.xp} xp). "
            f"DV {self.dv}{mod_dv} ({self.pv} pv), "
            f"CA {self.ca}, JP {self.jp}, "
            f"MV {self.movimento}, MO {self.moral}. "
            f"Ataques: {ataques_str}"
        )
    
    def __setattr__(self, nome, valor):
        # Se o atributo estiver no mapeamento, sincroniza com _dados
        if 'mapa_de_atributos' in self.__dict__ or nome in ('_dados', 'mapa_de_atributos'):
            # Durante __init__, permite setar normalmente
            super().__setattr__(nome, valor)
        else:
            # Verifica se é um atributo mapeado em português
            chave_original = None
            for k, v in self.mapa_de_atributos.items():
                if v == nome:
                    chave_original = k
                    break

            # Atualiza _dados se encontrar correspondência
            if chave_original and '_dados' in self.__dict__:
                self._dados[chave_original] = valor

            super().__setattr__(nome, valor)
    
    
    #? Métodos privados
    def _retornar_dv(self, dv_original: str):
        if dv_original == '½':
            return 0.5
        return converter_para_numero(dv_original)


    def _retornar_mod_dv(self, dv_original: str):
        if not dv_original:
            return 0
        return converter_para_numero(dv_original)


    def _retornar_ca(self, ca: str):
        convertido = converter_para_numero(ca)

        if isinstance(convertido, int):
            return convertido
        
        if '/' in ca:
            cas = ca.split('/')
            return list(int(c) for c in cas)

        return ca


    def _retornar_dano_causado(self, rolamento_ataque, rolamento_dano, mod_aditivo, mod_multiplicativo):
        dano_total = 0

        if rolamento_ataque.sucesso:
            dano_total = rolamento_dano.total if not rolamento_ataque.critico else (
                rolamento_dano.total_dados * 2) + rolamento_dano.modificador

            dano_total += mod_aditivo
            dano_total *= mod_multiplicativo

        return dano_total


    def _retornar_encontro(self, texto_encontro: str):
        if texto_encontro == '-' or isinstance(texto_encontro, type(None)):
            return 0
        
        if 'd' in texto_encontro:
            return rolar_dado_notacao(texto_encontro)
        
        return int(texto_encontro)


    def _retornar_tesouro(self, tipo: str):
        return TesouroAleatorio(tipo)


    #? Métodos públicos
    def calcular_vida(self):
        """Recalcula a vida, para o caso de mudar o HD da criatura
        """
        vida_base = 5 * self.dv
        
        self.pv = vida_base + self.dv_bonus


    def escolher_ca(self, indice: int = 0):
        """Escolhe qual a armadura do monstro atual caso haja mais de uma opção

        Args:
            indice (int, optional): Qual o índice da armadura na lista de opções. Defaults to 0.

        Raises:
            TypeError: Caso a armadura não possua opções a serem escolhidas indica isso
        """
        if isinstance(self.ca, list):
            self.ca = self.ca[indice]
        else:
            raise TypeError(f'CA não pode ser escolhida, valor de CA: {self.ca}')


    def escolher_arma(self, arma: dict | str):
        """Escolhe a arma que o monstro usa, caso ele tenha essa opção

        Args:
            arma (dict | str): A arma a ser usada, seja o dicionário da mesma ou o nome
        """
        if isinstance(arma, str):
            arma = buscar(DATA.EQUIPAMENTOS, arma, 'name')

        for ataque in self.ataques:
            if ataque.get('weapon') == True:
                nome = arma.get('name')

                ataque['damage'] = arma.get('damage')
                ataque['damage_description'] = f'{ataque['damage']}{converter_para_texto(ataque['damage_bonus'])}'
                ataque['description'] = nome
                ataque['text'] = f"{ataque['times']} × {nome} {converter_para_texto(ataque['ba'])} ({ataque['damage_description']})"


    def rolar_moral(self):
        """Rola a moral do monstro

        Returns:
            ResultadoMoral: tupla nomeada -> 'total' (o valor total do 2d6), 'rolamento' (os dois dados rolados) e 'sucesso' (bool, se passou no teste)
        """
        ResultadoMoral = namedtuple('ResultadoMoral', ('total', 'dados', 'sucesso'))

        dados = Dado(2, 6)
        rolamento = dados.rolamento
        total = dados.total
        sucesso = total <= self.moral

        return ResultadoMoral(total, rolamento, sucesso)


    def rolar_ataque(self, indice_ataque: int = 0, modificador_ataque: int = 0, ca_alvo: int = 10):
        """Executa o rolamento de ataque contra a ac de um alvo

        Args:
            indice_ataque (int, optional): O índice do ataque na lista de ataques. Defaults to 0.
            modificador_ataque (int, optional): Modificador situacional ao ataque. Defaults to 0.
            ca_alvo (int, optional): A CA do alvo para determinar se houve sucesso. Defaults to 10.

        Raises:
            ValueError: Quando se escolhe um índice de ataque inapropriado

        Returns:
            Ataque: named tuple: .rolamento (quanto que rolou no dado), .rolamento_modificado (o valor total do ataque), .resultado (tupla, indicando se acertou ou errou), .critico (tupla, indicando se foi acerto ou falha crítica)
        """
        if indice_ataque >= len(self.ataques):
            raise IndexError(
                f'O índice {indice_ataque} não está disponível. Valor máximo: {len(self.ataques) - 1}')
        ba = self.ataques[indice_ataque].get('ba')
        
        return jogada_ataque(ba, ca_alvo, modificador_ataque)


    def rolar_dano(self, indice_ataque: int = 0):
        """Rola o dano de um determinado ataque, escolhido dentre os ataques do monstro

        Args:
            indice_ataque (int, optional): Qual o índice do ataque escolhido. Defaults to 0.

        Raises:
            ValueError: Se o índice não está no alcance dos ataques

        Returns:
            ResultadoDano: namedtuple: .total (o dano total rolado), .total_dados (a soma dos dados rolados), .dados (os dados invididuais), .modificador (o modificador aplicado no dano)
        """
        ResultadoDano = namedtuple('ResultadoDano', ('total', 'total_dados', 'dados', 'modificador'))

        if indice_ataque >= len(self.ataques):
            raise IndexError(
                f'O índice {indice_ataque} não está disponível. Valor máximo: {len(self.ataques) - 1}')

        qtd, face = self.ataques[indice_ataque]['damage'].split('d')
        dado = Dado(int(qtd), int(face))
        mod = self.ataques[indice_ataque]['damage_bonus']
        mod = mod if mod else 0

        dano_total = dado.total + mod
        dano_total = dano_total if dano_total > 0 else 1

        return ResultadoDano(dano_total, dado.total, dado.rolamento, mod)


    def atacar(self,
               alvo,
               indice_ataque: int = 0,
               mod_ataque: int = 0,
               mod_dano_aditivo: int = 0,
               mod_dano_multiplicativo: int = 1
               ):
        """Ataca um alvo específico e reduz os pvs dele caso haja um acerto

        Args:
            alvo (Monstro | PC): O alvo do ataque, que tenha as propriedades .ca e .pv_atual
            indice_ataque (int, optional): O índice do ataque na lista de .ataques. Defaults to 0.
            mod_ataque (int, optional): Qual o modificador circunstancial do ataque. Defaults to 0.
            mod_dano_aditivo (int, optional): Modificado aditivo de dano. Defaults to 0.
            mod_dano_multiplicativo (int, optional): Modificador multiplicado do dano final. Defaults to 1.

        Returns:
            dict: Um dicionário com as chaves para referência do ataque
        """
        rolamento_ataque = self.rolar_ataque(indice_ataque, mod_ataque, alvo.ca)
        rolamento_dano = self.rolar_dano(indice_ataque)

        dano_total = self._retornar_dano_causado(rolamento_ataque, rolamento_dano, mod_dano_aditivo, mod_dano_multiplicativo)
        alvo.ferir(dano_total)

        resultado = {
            'dano final': dano_total,
            'ataque': rolamento_ataque,
            'dano': rolamento_dano,
        }

        return resultado


    def rolar_tesouro(self, covil: bool = False, tesouro_rapido: bool = False):
        tipo = self.tesouro if not covil else self.tesouro_covil

        if not tipo:
            return 'Sem tesouro'

        tesouro = self._retornar_tesouro(tipo[0])

        if len(tipo) == 3:
            operacao = tipo[1]

            if operacao == 'x':
                repeticao = eval(tipo[2])
                for _ in range(1, repeticao):
                    tesouro + self._retornar_tesouro(tipo[0])

            if operacao == '+':
                tesouro + self._retornar_encontro(tipo[2])


        if tesouro_rapido:
            return tesouro.tesouro_rapido
        
        return tesouro.tesouro


    def rolar_quantidade_encontro(self, covil: bool = False):
        dado = self.encontros if not covil else self.encontros_covil

        return self._retornar_encontro(dado)




from collections import namedtuple
from RPG import d20, d6


Rolagem = namedtuple('Rolagem', ('rolamento', 'numero_alvo', 'sucesso'))
Ataque = namedtuple('Ataque', ('rolamento', 'total', 'sucesso', 'critico'))
ChanceEm6 = namedtuple('chance_em_6', ('rolamento', 'sucesso'))


def teste_atributo(valor_atributo: int, modificador: int = 0):
    numero_alvo = valor_atributo + modificador

    rolamento = d20()
    resultado = rolamento <= numero_alvo

    if rolamento == 20:
        resultado = False

    if rolamento == 1:
        resultado = True
    
    return Rolagem(rolamento, numero_alvo, resultado)


def jogada_protecao(valor_protecao: int, modificador: int = 0):
    return teste_atributo(valor_protecao, modificador)


def jogada_ataque(ba: int, ca_alvo: int, modificador: int = 0, alcance_crit: int = 20):
    rolamento = d20()
    rolamento_modificado = rolamento + modificador + ba
    resultado = rolamento_modificado > ca_alvo
    critico = rolamento >= alcance_crit or rolamento == 1

    return Ataque(rolamento, rolamento_modificado, resultado, critico)

def x_em_d6(x: int):
    """Rola a chance em d6

    Args:
        x (int): A chance de rolamento bem sucedido, coloque o valor maior. Exemplo: 1-2 em 1d6 vai receber 2 como parâmetro

    Returns:
        ChanceEm6: namedtuple -> .rolamento (int, o dado rolado), .sucesso (bool, se teve sucesso)
    """
    rolamento = d6()
    resultado = rolamento <= x

    return ChanceEm6(rolamento, resultado)



#? Arquivo gerado a partir de livros.json

livros = [{'id': 'lb1',
  'title': 'Livro I: Regras Básicas',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/lb1.json'},
 {'id': 'lb2',
  'title': 'Livro II: Regras Expandidas',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/lb2.json'},
 {'id': 'lb3',
  'title': 'Livro III: Monstros & Inimigos',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/lb3.json'},
 {'id': 'srd',
  'title': 'SRD: Documento de Referência',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/srd.json'},
 {'id': 'arkhi',
  'title': 'Cenário de Campanha: ARKHI',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/arkhi.json'},
 {'id': 'valansia',
  'title': 'Cenário de Campanha: Heróis de Valansia',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/valansia.json'},
 {'id': 'legiao',
  'title': 'Cenário de Campanha: Legião - A Era da Desolação',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/legiao.json'},
 {'id': 'cl1',
  'title': 'CL1: O Forte das Terras Marginais',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/cl1.json'},
 {'id': 'tu1',
  'title': 'TU1: A Última Caravana do Outono',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/tu1.json'},
 {'id': 'tu2',
  'title': 'TU2: A Tumba dos Reis Serpentes',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/tu2.json'},
 {'id': 'nc1',
  'title': 'NC1: O Templo Perdido de Thur-Amon',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/nc1.json'},
 {'id': 'nc2',
  'title': 'NC2: O Monte do Desbrio',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/nc2.json'},
 {'id': 'ja1',
  'title': 'JA1: O Santuário Esquecido',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/ja1.json'},
 {'id': 'odday11',
  'title': 'OD Day 2011 Remaster: A Árvore Morta dos Kobolds Infernais',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday11.json'},
 {'id': 'odday12',
  'title': 'OD Day 2012 Remaster: O Templo dos Desmortos',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday12.json'},
 {'id': 'odday13',
  'title': 'OD Day 2013 Remaster: A Seita do Abismo',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday13.json'},
 {'id': 'odday14',
  'title': 'OD Day 2014 Remaster: A Fortaleza Sombria',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday14.json'},
 {'id': 'odday15',
  'title': 'OD Day 2015 Remaster: Fuga nas Terras Bárbaras',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday15.json'},
 {'id': 'odday16',
  'title': 'OD Day 2016 Remaster: Maldição na Garra da Mantícora',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday16.json'},
 {'id': 'odday17',
  'title': 'OD Day 2017 Remaster: Caçada nas Terras Gélidas',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday17.json'},
 {'id': 'odday18',
  'title': 'OD Day 2018 Remaster: A Sombra que Veio do Mar',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday18.json'},
 {'id': 'odday21',
  'title': 'OD Day 2021: Os Salões Eternos de Thalaknasus',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday21.json'},
 {'id': 'odday22',
  'title': 'OD Day 2022: Os Apóstatas do Deus Polvo',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday22.json'},
 {'id': 'odday23',
  'title': 'OD Day 2023: O Covil Flamejante de Rezingar',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday23.json'},
 {'id': 'odday24',
  'title': 'OD Day 2024: Incursão ao Charco das Feras',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/odday24.json'},
 {'id': 'ermos',
  'title': 'Guia de Campanha: Ermos',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/ermos.json'},
 {'id': 'sdg',
  'title': 'Guia de Campanha: Senhores da Guerra',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/sdg.json'},
 {'id': 'legiao-od2',
  'title': 'Jogando Legião com Old Dragon 2',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/legiao-od2.json'},
 {'id': 'monstros-legiao',
  'title': 'Monstros & Inimigos II: Nós Somos a Legião',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/monstros-legiao.json'},
 {'id': 'kit-do-jogador',
  'title': 'Kit do Jogador',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/kit-do-jogador.json'},
 {'id': 'kit-do-mestre',
  'title': 'Kit do Mestre',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/kit-do-mestre.json'},
 {'id': 'caveira',
  'title': 'A Marca da Caveira',
  'type': 'DigitalItem',
  'url': 'https://olddragon.com.br/livros/caveira.json'}]



from ..od2.data import DATA

def buscar_raca(busca: str, chave: str = 'id'):
    """Busca uma raça na api
    Args:
        busca (str): o id da raça a ser buscado. Pode ser também o nome
        chave (str, optional): se a busca é por id ou nome. Defaults to 'id'.

    Returns:
        dict: o dicionário com os dados da raça
    """

    resultado = [raca for raca in DATA.RACAS if raca[chave] == busca]
    return resultado[0]


class Raca:
    def __init__(self, data: str):
        self.data = data

    def __str__(self):
        return f'Raca(data["id"]="{self.data['id']}")'

    def __repr__(self):
        return f'Raca(data["id"]="{self.data['id']}")'


    #! Propriedades
    @property
    def movimento(self):
        return self.data['movement']

    @property
    def infravisao(self):
        return self.data['infravision']

    @property
    def alinhamento(self):
        return self.data['alignment_tendency']


from typing import Literal
from RPG import rolar_dado_notacao

from ..utils.gera_atributos import Atributos
from ..helpers import Atributo, AtributoBase, Moedas
from .raca import buscar_raca, Raca
from .classe import buscar_classe, Classe


ALINHAMENTOS = Literal[
    'ordeiro',
    'neutro',
    'caótico'
]


class Personagem:
    def __init__(self,
                 nome: str,
                 atributos: tuple | Atributos,
                 raca: dict | str,
                 classe: dict | str,
                 alinhamento: ALINHAMENTOS,
                 xp: int = 0,
                 vida: list | None = None,
                 vida_atual: int | None = None,
                 modificadores: dict | None = None,
                 moedas: dict | None = None,
                 ):
        
        self._atributos = atributos if type(atributos) is Atributos else Atributos(*atributos)
        self._raca = raca
        self._classe = classe
        self._xp = xp
        self._vida = vida
        self._vida_atual = vida_atual

        self.nome = nome
        self.alinhamento = alinhamento.capitalize()
        self.moedas = Moedas(moedas) if moedas else Moedas(0, 0, 0)
        self.modificadores = modificadores if modificadores else dict()

        self._ajustar_raca()
        self._ajustar_classe()
        self._ajustar_lista_vida()


    def __repr__(self) -> str:
        return f"Personagem(classe='{self.classe.data['name']}', raça='{self.raca.data['name']}', alinhamento='{self.alinhamento}', nivel={self.nivel})"

    def __str__(self):
        return f'{self.nome}: {self.classe.data['name']} {self.raca.data['name']} {self.alinhamento}, nível {self.nivel}'


    #! Propriedades
    @property
    def _modificadores_vida(self):
        lista = self.classe.vida_bonus

        for i in range(10):
            lista[i] += self.CON.modificador

        return lista

    @property
    def FOR(self):
        return Atributo(self._atributos.FOR)

    @property
    def DES(self):
        return Atributo(self._atributos.DES)

    @property
    def CON(self):
        return Atributo(self._atributos.CON)

    @property
    def INT(self):
        return Atributo(self._atributos.INT)

    @property
    def SAB(self):
        return Atributo(self._atributos.SAB)

    @property
    def CAR(self):
        return Atributo(self._atributos.CAR)
    

    @property
    def raca(self):
        return self._raca
    
    @property
    def classe(self):
        return self._classe
    
    @property
    def xp(self):
        return self._xp
    
    @property
    def nivel(self):
        xps = self.classe.xp

        xp_proximo_nivel = next((i for i in xps if i > self.xp), None)
        nivel = xps.index(xp_proximo_nivel) if xp_proximo_nivel else 15

        return nivel
    

    @property
    def JP(self):
        mod = self._retornar_modificador('jp')
        return self.classe.jp[self.nivel] + mod

    @property
    def JPC(self):
        mod = self._retornar_modificador('jpc')
        return AtributoBase(self.JP + self.CON.modificador + mod)

    @property
    def JPD(self):
        mod = self._retornar_modificador('jpd')
        return AtributoBase(self.JP + self.DES.modificador + mod)

    @property
    def JPS(self):
        mod = self._retornar_modificador('jps')
        return AtributoBase(self.JP + self.SAB.modificador + mod)
    

    @property
    def BA(self):
        return self.classe.ba[self.nivel]

    @property
    def BAD(self):
        return self.BA + self.DES.modificador

    @property
    def BAC(self):
        return self.BA + self.FOR.modificador


    @property
    def vida(self):
        mod_vida = self._modificadores_vida[:self.nivel]
        lista_resultante = [max(a + b, 1) for a, b in zip(mod_vida, self._vida)]

        return sum(lista_resultante)
    
    @property
    def vida_atual(self):
        self._vida_atual = self._vida_atual if type(
            self._vida_atual) is int else self.vida
        self._vida_atual = min(max(self._vida_atual, 0), self.vida)

        return self._vida_atual

    @vida_atual.setter
    def vida_atual(self, valor: int):
        self._vida_atual = valor

    
    @property
    def movimento(self):
        return self.raca.movimento
    
    @property
    def habilidades(self):
        habilidades = []

        if self.raca.infravisao:
            habilidades += [{
                'name': 'Infravisão',
                'description': f'infravisão de {self.raca.infravisao} metros'
            }]

        de_raca = self.raca.data['abilities']

        de_classe = [habilidade for habilidade in self.classe.data['abilities']
                     if habilidade['level'] <= self.nivel]

        return habilidades + de_raca + de_classe

    @property
    def carga_maxima(self):
        modificador = self._retornar_modificador('carga')

        return max(self.FOR.valor, self.CON.valor) + modificador


    #! Métodos inicializadores ou privados
    def _ajustar_raca(self):
        if type(self._raca) is str:
            self._raca = buscar_raca(self._raca)

        self._raca = Raca(self._raca)

    def _ajustar_classe(self):
        if type(self._classe) is str:
            self._classe = buscar_classe(self._classe)

        self._classe = Classe(self._classe)

    def _ajustar_lista_vida(self):
        # cria a lista base para o caso de já não ter valores rolados
        if not self._vida:
            self._vida = [self.classe.vida[0]]

        # retira o excesso de dados, caso tenha mais que o nível
        if len(self._vida) > self.nivel:
            self._vida = self._vida[0:self.nivel]

    def _retornar_modificador(self, chave: str):
        modificador = 0
        if chave in self.modificadores.keys():
            modificador += sum(self.modificadores[chave].values())

        return modificador

    #! Métodos
    def aplicar_xp(self, valor_recebido: int):
        modificador = 1

        if self._raca == 'humano':
            modificador += .1

        if self._raca == 'meio-elfo':
            modificador += .05

        valor_recebido = int(valor_recebido * modificador)
        self._xp += valor_recebido

    def rolar_vida(self):
        """
        Rola a vida de todos os níveis não rolados
        """
        while len(self._vida) < self.nivel:
            vida_atual = self.classe.vida[len(self._vida)]
            vida_atual = rolar_dado_notacao(vida_atual) if type(
                vida_atual) is str else vida_atual

            self._vida.append(vida_atual)

    def drenar_nivel(self, qtd_drenado: int):
        # TODO
        pass


from RPG.dado import Dado

def rolar_renda_inicial():
    """
    Gera a renda inicial

    Returns:
        tupla: retorna uma tupla com o total e os dados rolados
    """

    dado = Dado(3, 6)
    
    return dado.total * 10, dado.rolamento



PREFIXO = 'data/api/'

URL_BASE = 'https://olddragon.com.br/'
URL_ESPECIFICAS = {
    'classes': 'classes.json',
    'equipamentos': 'equipamentos.json',
    'livros': 'livros.json',
    'magias': 'magias.json',
    'monstros': 'monstros.json',
    'raças': 'racas.json'
}



from collections import namedtuple

ModAtributos = namedtuple('ModAtributo', ('mod', 'circulo_1', 'circulo_2', 'circulo_3'))

ATRIBUTOS = ('FOR', 'DES', 'CON', 'INT', 'SAB', 'CAR')
ATRIBUTOS_EXTENSO = ('Força', 'Destreza', 'Constituição', 'Inteligência', 'Sabedoria', 'Carisma')
ATRIBUTOS_MOD = {
    3: ModAtributos(-3, 0, 0, 0),
    5: ModAtributos(-2, 0, 0, 0),
    8: ModAtributos(-1, 0, 0, 0),
    12: ModAtributos(0, 0, 0, 0),
    14: ModAtributos(1, 1, 0, 0),
    16: ModAtributos(2, 1, 1, 0),
    18: ModAtributos(3, 2, 1, 1),
    20: ModAtributos(4, 2, 2, 1),
}



from dataclasses import dataclass
from typing import Literal


moedas = Literal['pc', 'pp', 'po']

CAMBIO = {
    'pc': 1,
    'pp': 10,
    'po': 100
}

PESO = 1/100


def converter_moedas(valor: int, de: moedas = 'pp', para: moedas = 'po'):
    maior = CAMBIO[de] > CAMBIO[para]
    atual, resto = 0, 0

    if maior:
        atual = valor * (CAMBIO[de] / CAMBIO[para])

    else:
        atual = valor / (CAMBIO[para] / CAMBIO[de])
        resto = valor % (CAMBIO[para] / CAMBIO[de])

    return int(atual), int(resto)


@dataclass
class Moedas:
    pc: int = 0
    pp: int = 0
    po: int = 0

    @property
    def carga(self):
        volume = self.pc + self.pp + self.po
        return int(volume * PESO)
    
    @property
    def saldo_po(self):
        pc = converter_moedas(self.pc, 'pc', 'po')
        pp = converter_moedas(self.pp, 'pp', 'po')

        po = 0
        pc = pc[0] + pc[1] / 100
        pp = pp[0] + pp[1] / 10

        troco = pc + pp

        if troco > 100:
            po = int(troco/100)
            troco = troco % 100

        return self.po + po + troco
    

    #! Métodos de conversão
    def pc_para_po(self, valor: int):
        # se o valor for maior, só converte o que pode
        if valor > self.pc:
            valor = self.pc
        
        self.pc -= valor

        po, pc = converter_moedas(valor, 'pc', 'po')
        print(po, pc)

        self.pc += pc
        self.po += po

    def pp_para_po(self, valor: int):
        # se o valor for maior, só converte o que pode
        if valor > self.pp:
            valor = self.pp

        self.pp -= valor

        po, pp = converter_moedas(valor, 'pp', 'po')

        self.pp += pp
        self.po += po

    def pp_para_pc(self, valor: int):
        # se o valor for maior, só converte o que pode
        if valor > self.pp:
            valor = self.pp

        self.pp -= valor

        pc, _ = converter_moedas(valor, 'pp', 'pc')

        self.pc += pc

    def pc_para_pp(self, valor: int):
        # se o valor for maior, só converte o que pode
        if valor > self.pc:
            valor = self.pc

        self.pc -= valor

        pp, pc = converter_moedas(valor, 'pc', 'pp')

        self.pc += pc
        self.pp += pp

    def po_para_pp(self, valor: int):
        # se o valor for maior, só converte o que pode
        if valor > self.po:
            valor = self.po

        self.po -= valor

        pp, _ = converter_moedas(valor, 'po', 'pp')

        self.pp += pp

    def po_para_pc(self, valor: int):
        # se o valor for maior, só converte o que pode
        if valor > self.po:
            valor = self.po

        self.po -= valor

        pc, _ = converter_moedas(valor, 'po', 'pc')

        self.pc += pc

    #! Outros métodos
    def otimizar_carga(self):
        """Reduz a quantidade de moedas carregadas, mantendo o saldo
        """
        self.pc_para_pp(self.pc)
        self.pp_para_po(self.pp)

    def comprar(self, valor_pc: int):
        """Compra um item, com a base do valor em peças de cobre, que é o padrão
        da API. Retorna o valor final otimizado

        Args:
            valor_pc (int): o valor do item a ser comprado
        """
        if converter_moedas(valor_pc, 'pc', 'po')[0] < self.saldo_po:
            self.po_para_pc(self.po)
            self.pp_para_pc(self.pp)

            self.pc -= valor_pc

            self.otimizar_carga()
        
        else:
            print('sem saldo')
        



#? Arquivo gerado a partir de personagens.json

personagens = [{'ac': 12,
  'alignment': 'caotico',
  'appearance': 'Alto (Corpo); Bem afeiçoado (Cabelos); Jovem (Geral)',
  'bac': -1,
  'background': 'Nascido numa aldeia nas montanhas em uma família adotiva, '
                'ficou órfão após de um ataque de orcs e goblins. Tornou-se '
                'aventureiro para matar a curiosidade sobre o mundo.',
  'bad': 3,
  'carisma': 7,
  'character_class_name': 'Arqueiro',
  'character_class_url': 'https://olddragon.com.br/classes/arqueiro.json',
  'character_race_name': 'Elfo',
  'character_race_url': 'https://olddragon.com.br/racas/elfo.json',
  'constituicao': 7,
  'created_at': '2024-08-02T14:02:24.130-03:00',
  'current_movement': 9,
  'destreza': 15,
  'experience_points': 2500,
  'forca': 4,
  'health_points': 9,
  'id': '05aaee89-74a0-4eef-ae90-3bba0cd912e1',
  'infravision': 18,
  'injuries': 0,
  'inteligencia': 13,
  'inventory_items': [{'arrow': False,
                       'bolt': False,
                       'bolt_small': False,
                       'bonus_ba': None,
                       'bonus_ca': None,
                       'bonus_damage': None,
                       'concept': 'weapon',
                       'cost': 2500,
                       'counter_attack': False,
                       'damage': None,
                       'damage_type': 'none',
                       'description': 'Média, Disparo (30), Duas Mãos',
                       'equippable?': True,
                       'equipped': True,
                       'increases_load_by': None,
                       'magic_item': False,
                       'mounted': False,
                       'name': 'Arco Curto',
                       'polearm': False,
                       'position': 1,
                       'quantity': 1,
                       'recharge': False,
                       'shoot_range': 30,
                       'sum_weight_in_grams': 2000,
                       'throw_range': None,
                       'total_cost': 2500,
                       'two_handed': True,
                       'versatile': False,
                       'weight_in_grams': None,
                       'weight_in_load': 2,
                       'wooden': False},
                      {'arrow': False,
                       'bolt': False,
                       'bolt_small': False,
                       'bonus_ba': None,
                       'bonus_ca': None,
                       'bonus_damage': None,
                       'concept': 'container',
                       'cost': 100,
                       'counter_attack': False,
                       'damage': None,
                       'damage_type': 'none',
                       'description': 'De couro, rígida para até 40 flechas ou '
                                      '80 virotes. Comporta até 150 moedas.',
                       'equippable?': False,
                       'equipped': False,
                       'increases_load_by': None,
                       'magic_item': False,
                       'mounted': False,
                       'name': 'Aljava',
                       'polearm': False,
                       'position': 2,
                       'quantity': 1,
                       'recharge': False,
                       'shoot_range': None,
                       'sum_weight_in_grams': 500,
                       'throw_range': None,
                       'total_cost': 100,
                       'two_handed': False,
                       'versatile': False,
                       'weight_in_grams': 500,
                       'weight_in_load': None,
                       'wooden': False},
                      {'arrow': True,
                       'bolt': False,
                       'bolt_small': False,
                       'bonus_ba': None,
                       'bonus_ca': None,
                       'bonus_damage': None,
                       'concept': 'weapon',
                       'cost': 1,
                       'counter_attack': False,
                       'damage': '1d6',
                       'damage_type': 'piercing',
                       'description': 'Perfurante, Madeira. Peso e preço pelo '
                                      'conjunto de 20 unidades.',
                       'equippable?': True,
                       'equipped': True,
                       'increases_load_by': None,
                       'magic_item': False,
                       'mounted': False,
                       'name': 'Flecha de Caça',
                       'polearm': False,
                       'position': 3,
                       'quantity': 20,
                       'recharge': False,
                       'shoot_range': None,
                       'sum_weight_in_grams': 0,
                       'throw_range': None,
                       'total_cost': 20,
                       'two_handed': False,
                       'versatile': False,
                       'weight_in_grams': None,
                       'weight_in_load': None,
                       'wooden': True},
                      {'arrow': False,
                       'bolt': False,
                       'bolt_small': False,
                       'bonus_ba': None,
                       'bonus_ca': None,
                       'bonus_damage': None,
                       'concept': 'weapon',
                       'cost': 200,
                       'counter_attack': False,
                       'damage': '1d4',
                       'damage_type': 'piercing',
                       'description': 'Pequena, Perfurante, Arremesso (9)',
                       'equippable?': True,
                       'equipped': True,
                       'increases_load_by': None,
                       'magic_item': False,
                       'mounted': False,
                       'name': 'Adaga',
                       'polearm': False,
                       'position': 4,
                       'quantity': 1,
                       'recharge': False,
                       'shoot_range': None,
                       'sum_weight_in_grams': 1000,
                       'throw_range': 9,
                       'total_cost': 200,
                       'two_handed': False,
                       'versatile': False,
                       'weight_in_grams': None,
                       'weight_in_load': 1,
                       'wooden': False}],
  'jpc': 4,
  'jpd': 8,
  'jps': 4,
  'languages': ['', '', ''],
  'level': 1,
  'max_hp': 9,
  'mod_carisma': -1,
  'mod_constituicao': -1,
  'mod_destreza': 2,
  'mod_forca': -2,
  'mod_inteligencia': 1,
  'mod_sabedoria': -1,
  'money_cp': 0,
  'money_gp': 1,
  'money_sp': 9,
  'movement_climb': 7,
  'movement_run': 18,
  'movement_swim': 4,
  'name': 'Adellas',
  'notes': '',
  'personality': 'Teimoso (Sobre todo o resto); Agitado (Sobre si mesmo); '
                 'Gentil (Sobre outras pessoas)',
  'picture': 'https://public.olddragon.com.br/lx0wrpi9av85i0fmm1rm7kc9a7df',
  'race_jp': 'jpd',
  'sabedoria': 8,
  'updated_at': '2025-01-31T11:54:40.561-03:00',
  'url': 'https://olddragon.com.br/personagens/05aaee89-74a0-4eef-ae90-3bba0cd912e1.json'},
 {'ac': 18,
  'alignment': 'neutro',
  'appearance': 'Baixo (Corpo); Cabelos curtos (Cabelos); Pálido (Geral)',
  'bac': 2,
  'background': 'Sua vida era tranquila e pacífica junto da sua família de '
                'fazendeiros, até que, há alguns anos, você foi vítima de uma '
                'epidemia de peste e foi forçado a aprender sozinho as '
                'técnicas de Anão Aventureiro (Guerreiro). Hoje, se aventura '
                'para trazer glória e fama para você.',
  'bad': 4,
  'campaign_name': 'Teste',
  'campaign_url': 'https://olddragon.com.br/campanhas/df0b14ba-c7d3-4324-86fc-7b4abe5bb478.json',
  'carisma': 15,
  'character_class_name': 'Anão Aventureiro',
  'character_class_url': 'https://olddragon.com.br/classes/anao-aventureiro.json',
  'character_race_name': 'Anão',
  'character_race_url': 'https://olddragon.com.br/racas/anao.json',
  'constituicao': 10,
  'created_at': '2024-08-28T14:09:42.403-03:00',
  'current_movement': 6,
  'destreza': 17,
  'experience_points': 50,
  'forca': 13,
  'health_points': 8,
  'id': '6bc9a4f2-f71e-4418-a604-52e547d359b8',
  'infravision': 18,
  'injuries': 2,
  'inteligencia': 3,
  'inventory_items': [{'arrow': False,
                       'bolt': False,
                       'bolt_small': False,
                       'bonus_ba': None,
                       'bonus_ca': None,
                       'bonus_damage': None,
                       'concept': 'weapon',
                       'cost': 600,
                       'counter_attack': False,
                       'damage': '1d8',
                       'damage_type': 'slashing',
                       'description': 'Média, Cortante.',
                       'equippable?': True,
                       'equipped': True,
                       'increases_load_by': None,
                       'magic_item': False,
                       'mounted': False,
                       'name': 'Machado',
                       'polearm': False,
                       'position': 1,
                       'quantity': 1,
                       'recharge': False,
                       'shoot_range': None,
                       'sum_weight_in_grams': 2000,
                       'throw_range': None,
                       'total_cost': 600,
                       'two_handed': False,
                       'versatile': False,
                       'weight_in_grams': None,
                       'weight_in_load': 2,
                       'wooden': False},
                      {'arrow': False,
                       'bolt': False,
                       'bolt_small': False,
                       'bonus_ba': None,
                       'bonus_ca': 1,
                       'bonus_damage': None,
                       'concept': 'shield',
                       'cost': 800,
                       'counter_attack': False,
                       'damage': None,
                       'damage_type': 'none',
                       'description': 'Leve, Mão Exclusiva, Madeira.',
                       'equippable?': True,
                       'equipped': True,
                       'increases_load_by': None,
                       'magic_item': False,
                       'mounted': False,
                       'name': 'Escudo',
                       'polearm': False,
                       'position': 2,
                       'quantity': 1,
                       'recharge': False,
                       'shoot_range': None,
                       'sum_weight_in_grams': 1000,
                       'throw_range': None,
                       'total_cost': 800,
                       'two_handed': False,
                       'versatile': False,
                       'weight_in_grams': None,
                       'weight_in_load': 1,
                       'wooden': False},
                      {'arrow': False,
                       'bolt': False,
                       'bolt_small': False,
                       'bonus_ba': None,
                       'bonus_ca': 4,
                       'bonus_damage': None,
                       'concept': 'armor',
                       'cost': 6000,
                       'counter_attack': False,
                       'damage': None,
                       'damage_type': 'none',
                       'description': 'Média, Metal.',
                       'equippable?': True,
                       'equipped': True,
                       'increases_load_by': None,
                       'magic_item': False,
                       'mounted': False,
                       'name': 'Cota de Malha',
                       'polearm': False,
                       'position': 3,
                       'quantity': 1,
                       'recharge': False,
                       'shoot_range': None,
                       'sum_weight_in_grams': 2000,
                       'throw_range': None,
                       'total_cost': 6000,
                       'two_handed': False,
                       'versatile': False,
                       'weight_in_grams': None,
                       'weight_in_load': 2,
                       'wooden': False},
                      {'arrow': False,
                       'bolt': False,
                       'bolt_small': False,
                       'bonus_ba': None,
                       'bonus_ca': None,
                       'bonus_damage': None,
                       'concept': 'weapon',
                       'cost': 200,
                       'counter_attack': False,
                       'damage': '1d4',
                       'damage_type': 'piercing',
                       'description': 'Pequena, Perfurante, Arremesso (9)',
                       'equippable?': True,
                       'equipped': True,
                       'increases_load_by': None,
                       'magic_item': False,
                       'mounted': False,
                       'name': 'Adaga',
                       'polearm': False,
                       'position': 4,
                       'quantity': 3,
                       'recharge': False,
                       'shoot_range': None,
                       'sum_weight_in_grams': 3000,
                       'throw_range': 9,
                       'total_cost': 600,
                       'two_handed': False,
                       'versatile': False,
                       'weight_in_grams': None,
                       'weight_in_load': 1,
                       'wooden': False}],
  'jpc': 6,
  'jpd': 8,
  'jps': 4,
  'languages': [],
  'level': 1,
  'max_hp': 10,
  'mod_carisma': 2,
  'mod_constituicao': 0,
  'mod_destreza': 3,
  'mod_forca': 1,
  'mod_inteligencia': -3,
  'mod_sabedoria': -1,
  'money_cp': 0,
  'money_gp': 40,
  'money_sp': 0,
  'movement_climb': 4,
  'movement_run': 12,
  'movement_swim': 3,
  'name': 'Hagnos',
  'notes': '',
  'personality': 'Otimista (Sobre todo o resto); Alerta (Sobre si mesmo); '
                 'Gentil (Sobre outras pessoas)',
  'picture': 'https://public.olddragon.com.br/ox4gmakn58mr4m0ne9qn5qqogq3b',
  'race_jp': 'jpc',
  'sabedoria': 8,
  'updated_at': '2025-02-01T12:41:18.298-03:00',
  'url': 'https://olddragon.com.br/personagens/6bc9a4f2-f71e-4418-a604-52e547d359b8.json'},
 {'ac': 12,
  'alignment': 'neutro',
  'appearance': 'Baixo (Corpo); Cabelos longos (Cabelos); Moreno (Geral)',
  'bac': 2,
  'background': 'Ainda criança, foi abandonado numa fazenda nos campos. '
                'Cresceu sob os cuidados de um Mago o qual lhe ensinou tudo o '
                'que ele sabe. Tornou-se aventureiro para vingar seu passado.',
  'bad': 1,
  'carisma': 13,
  'character_class_name': 'Mago',
  'character_class_url': 'https://olddragon.com.br/classes/mago.json',
  'character_race_name': 'Humano',
  'character_race_url': 'https://olddragon.com.br/racas/humano.json',
  'constituicao': 7,
  'created_at': '2024-10-09T10:15:23.846-03:00',
  'current_movement': 9,
  'destreza': 14,
  'experience_points': 0,
  'forca': 16,
  'health_points': 3,
  'id': 'e16b225d-3136-470d-b6c4-89dc9f4c411e',
  'infravision': 0,
  'injuries': 0,
  'inteligencia': 17,
  'inventory_items': [{'arrow': False,
                       'bolt': False,
                       'bolt_small': False,
                       'bonus_ba': None,
                       'bonus_ca': None,
                       'bonus_damage': None,
                       'concept': 'weapon',
                       'cost': 800,
                       'counter_attack': False,
                       'damage': '1d10',
                       'damage_type': 'slashing',
                       'description': 'Grande, Cortante, Duas Mãos, Haste',
                       'equippable?': True,
                       'equipped': True,
                       'increases_load_by': None,
                       'magic_item': False,
                       'mounted': False,
                       'name': 'Alabarda',
                       'polearm': True,
                       'position': 1,
                       'quantity': 1,
                       'recharge': False,
                       'shoot_range': None,
                       'sum_weight_in_grams': 3000,
                       'throw_range': None,
                       'total_cost': 800,
                       'two_handed': True,
                       'versatile': False,
                       'weight_in_grams': None,
                       'weight_in_load': 3,
                       'wooden': False},
                      {'arrow': False,
                       'bolt': False,
                       'bolt_small': False,
                       'bonus_ba': None,
                       'bonus_ca': 1,
                       'bonus_damage': None,
                       'concept': 'armor',
                       'cost': 500,
                       'counter_attack': False,
                       'damage': None,
                       'damage_type': 'none',
                       'description': 'Leve, Couro.',
                       'equippable?': True,
                       'equipped': True,
                       'increases_load_by': None,
                       'magic_item': False,
                       'mounted': False,
                       'name': 'Armadura Acolchoada',
                       'polearm': False,
                       'position': 2,
                       'quantity': 1,
                       'recharge': False,
                       'shoot_range': None,
                       'sum_weight_in_grams': 1000,
                       'throw_range': None,
                       'total_cost': 500,
                       'two_handed': False,
                       'versatile': False,
                       'weight_in_grams': None,
                       'weight_in_load': 1,
                       'wooden': False}],
  'jpc': 5,
  'jpd': 6,
  'jps': 6,
  'languages': ['', '', '', '', ''],
  'level': 1,
  'max_hp': 3,
  'mod_carisma': 1,
  'mod_constituicao': -1,
  'mod_destreza': 1,
  'mod_forca': 2,
  'mod_inteligencia': 3,
  'mod_sabedoria': 1,
  'money_cp': 0,
  'money_gp': 107,
  'money_sp': 0,
  'movement_climb': 7,
  'movement_run': 18,
  'movement_swim': 4,
  'name': 'Idildo',
  'notes': '',
  'personality': 'Idealista (Sobre todo o resto); Calmo (Sobre si mesmo); '
                 'Tímido (Sobre outras pessoas)',
  'picture': 'https://public.olddragon.com.br/ta8wd3d5kzs4pwpis58nsq7l1cky',
  'race_jp': 'jpc',
  'sabedoria': 14,
  'updated_at': '2025-02-18T08:22:04.188-03:00',
  'url': 'https://olddragon.com.br/personagens/e16b225d-3136-470d-b6c4-89dc9f4c411e.json'}]



from ...od2.data import DATA


def buscar_classe(busca: str, chave: str = 'id'):
    """Busca uma classe na api
    Args:
        busca (str): o id da classe a ser buscado. Pode ser também o nome
        chave (str, optional): se a busca é por id ou name. Defaults to 'id'.

    Returns:
        dict: o dicionário com os dados da classe
    """

    resultado = [raca for raca in DATA.CLASSES if raca[chave] == busca]
    return resultado[0]


class Classe:
    def __init__(self, data: str):
        self.data = data


    def __str__(self):
        return f'Classe(data["id"]="{self.data['id']}")'

    def __repr__(self):
        return f'Classe(data["id"]="{self.data['id']}")'


    #! Propriedades
    @property
    def vida(self):
        pv_base = self.data['hp']
        pv_nivel_alto = self.data['high_level_hp_bonus']

        resultado = [pv_base]
        for _ in range(9):
            resultado.append(f'1d{pv_base}')

        for _ in range(5):
            resultado.append(pv_nivel_alto)

        return resultado

    @property
    def xp(self):
        return self._retornar_por_nivel('xp')

    @property
    def ba(self):
        return self._retornar_por_nivel('ba')

    @property
    def jp(self):
        return self._retornar_por_nivel('jp')

    @property
    def vida_bonus(self):
        lista = [0 for _ in range(15)]

        if self.data['id'] == 'barbaro':
            for i in range(15):
                lista[i] += 2

        if self.data['id'] == 'anao-aventureiro':
            for i in range(2, 10):
                self.vida[i] = '1d12'

        if self.data['id'] == 'elfo-aventureiro':
            for i in range(15):
                lista[i] += 1

        return lista
    

    #! Métodos privados
    def _retornar_por_nivel(self, chave: str):
        db = self.data['levels']

        valores = [db[i][chave] for i in db if chave in db[i].keys()]
        valores.insert(0, 0)

        return valores




from typing import Literal
from collections import namedtuple
from RPG.dado import Dado

from ..data import DATA


LISTA_ATRIBUTOS = Literal['FOR', 'DES', 'CON', 'INT', 'SAB', 'CAR']

ESTILOS = Literal[
    'clássico',
    'aventureiro',
    'heroico',
    # 'duplo',
    # 'camponês',
    # 'distribuição',
    # 'racial'
]

Atributos = namedtuple('Atributos', ('FOR', 'DES', 'CON', 'INT', 'SAB', 'CAR'))

def rolar_atributos(estilo: ESTILOS = 'clássico'):
    """Gera os atributos conforme o estilo de rolagem

    Args:
        estilo (ESTILOS, optional): escolha entre os estilos de rolamento. Defaults to 'clássico'.

    Returns:
        GeradorAtributos: Uma classe de gerador para atribuir os atributos
    """
    return GeradorAtributos(estilo)


class GeradorAtributos:
    def __init__(self, estilo: ESTILOS):
        self._estilo = estilo

        self._iniciar_escolhidos()
        self.rolar()
        self.rolamento_original = tuple(self.rolamento)

    def __repr__(self) -> str:
        return f'{self.atributos}'
    

    #? Métodos privados
    def _iniciar_escolhidos(self):
        self._ordem_escolhidos = [None for _ in range(6)]

    def _checar_rolamento_classico_aventureiro(self):
        if self._estilo == 'clássico' or self._estilo == 'aventureiro':
            self.rolamento = list(Dado(3, 6) for _ in range(6))
            self._rolamento_totais = list(rolamento.total for rolamento in self.rolamento)

    def _checar_rolamento_heroico(self):
        if self._estilo == 'heroico':
            self.rolamento = [Dado(4, 6) for _ in range(6)]

            retira_menor = list(self.rolamento)
            for dado in retira_menor:
                dado.retirar_menor()

            self._rolamento_totais = [rolamento.total for rolamento in retira_menor]

    def _aplicar_resultado_na_ordem(self):
        if self._estilo == 'clássico' or self._estilo == 'duplo':
            self._ordem_escolhidos = list(atr for atr in DATA.ATRIBUTOS)

    def _retornar_resultado_final(self):
        if None not in self._ordem_escolhidos:
            self._aplicar_lista_final()

        else:
            self._retorna_lista_incompleta()

    def _aplicar_lista_final(self):
        final = []
        for i in DATA.ATRIBUTOS:
            indice = self._ordem_escolhidos.index(i)
            final.append(self._rolamento_totais[indice])

        self.atributos = Atributos(*final)

    def _retorna_lista_incompleta(self):
        atributos_disponiveis = list(i for i in DATA.ATRIBUTOS if i not in self._ordem_escolhidos)
        valores_disponiveis = list(str(self._rolamento_totais[i]) for i, x in enumerate(
            self._ordem_escolhidos) if not x)

        self.atributos = f'Atributos não escolhidos: {", ".join(
            atributos_disponiveis)}. Valores disponíveis: {", ".join(valores_disponiveis)}'

    def _encontrar_indice_livre(self, valor_escolhido):
        for i, (valor, atributo) in enumerate(zip(self._rolamento_totais, self._ordem_escolhidos)):
            if valor == valor_escolhido and atributo is None:
                return i
            
        return None

    #? Métodos públicos
    def rolar(self):
        self._iniciar_escolhidos()
        self._checar_rolamento_classico_aventureiro()
        self._checar_rolamento_heroico()

        self._aplicar_resultado_na_ordem()
        self._retornar_resultado_final()

    def atribuir_atributo(self, atributo: LISTA_ATRIBUTOS, valor: int):
        indice = self._encontrar_indice_livre(valor)

        if indice is not None:
            self._ordem_escolhidos[indice] = atributo
        else:
            print(f'Não há {valor} livre. Use .zerar_atributo() ou .trocar_atributo() para zerar o reatribuir')

        self._retornar_resultado_final()

    def zerar_atributo(self, atributo: LISTA_ATRIBUTOS):
        if atributo in self._ordem_escolhidos:
            indice = self._ordem_escolhidos.index(atributo)
            self._ordem_escolhidos[indice] = None
        
        self._retornar_resultado_final()

    def trocar_atributos(self, atributo1: LISTA_ATRIBUTOS, atributo2: LISTA_ATRIBUTOS):
        if atributo1 in self._ordem_escolhidos and atributo2 in self._ordem_escolhidos:
            indice1 = self._ordem_escolhidos.index(atributo1)
            indice2 = self._ordem_escolhidos.index(atributo2)

            self._ordem_escolhidos[indice2] = atributo1
            self._ordem_escolhidos[indice1] = atributo2

        self._retornar_resultado_final()
